#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "fitsio.h"
#include "pil.h"
#include "headas.h"
#include "headas_error.h"
#include "heasp.h"
#include "headas_copykeys.h"

#define MAXFLD 999
#define MAXMSG 256

typedef enum sprbnarf_error_code {
     BASE_ERROR = 2000,
     PARSE_ERROR
} sprbnarf_error_code;

       

/*
HISTORY
-------
  Version 1.0 written by kaa, NASA/GSFC, February 2008
*/

#define TOOLSUB sprbnarf
/* headas_main() requires that TOOLSUB be defined first */
#include "headas_main.c"

/* Function Prototypes */
int sprbnarf (void);
int sprbnarf_getpar (char* infile, char* binfile, int* binfact, char* outfile);
int sprbnarf_work (char* infile, char* binfile, int binfact, char* outfile);

/*---------------------------------------------------------------------------*/
int sprbnarf (void)
{
/*  Rebin an ARF file
*/

    char infile[PIL_LINESIZE], binfile[PIL_LINESIZE];
    char outfile[PIL_LINESIZE];
    int binfact; 
    int status;

    static char taskname[80] = "sprbnarf";
    static char version[8] = "1.00";

    /* Register taskname and version. */

    set_toolname(taskname);
    set_toolversion(version);

    /*  get input parameters */
    status = sprbnarf_getpar (infile, binfile, &binfact, outfile);

    /* call work function to do the rebinning */
    if (!status)
      status = sprbnarf_work (infile, binfile, binfact, outfile);

    return(status);
}
/*---------------------------------------------------------------------------*/
int sprbnarf_getpar(
    char* infile,     /* O - Name of input ARF file */ 
    char* binfile,    /* O - name of optional file specifying binning */ 
    int* binfact,     /* O - binning factor (if binfile not used) */ 
    char* outfile     /* O - name of output FITS file */
    )

/*  read input parameters for the sprbnarf task from the .par file */
{
    int status=0;
    char msg[MAXMSG];

    if ((status = PILGetFname("infile", infile))) {
      sprintf(msg, "Error reading the 'infile' parameter.");
      HD_ERROR_THROW(msg,status);
      return(status);
    }

    if ((status = PILGetFname("binfile", binfile))) {
      sprintf(msg, "Error reading the 'binfile' parameter.");
      HD_ERROR_THROW(msg,status);
      return(status);
    }

    if ( strcmp(binfile, "") == 0 ) {
      if ((status = PILGetInt("binfact", binfact))) {
	sprintf(msg, "Error reading the 'binfact' parameter.");
	HD_ERROR_THROW(msg,status);
	return(status);
      }
    }

    if ((status = PILGetString("outfile", outfile))) {
      sprintf(msg, "Error reading the 'outfile' parameter.");
      HD_ERROR_THROW(msg,status);
      return(status);
    }

    return(status);
}
/*---------------------------------------------------------------------------*/
int sprbnarf_work(
    char* infile,     /* I - Name of input ARF file */ 
    char* binfile,    /* I - name of ASCII binning factor file */ 
    int   binfact,    /* I - value of binning factor (if binfile not used) */ 
    char* outfile     /* I - name of output FITS file */
    )

{

  int status=0;
  int i, n;
  struct BinFactors *binning;
  int *groupArray;
  struct ARF *inputARF;
  struct ARF *outputARF;
  fitsfile *inptr, *outptr;

  /* read in the input ARF */

  if ( fits_open_file(&inptr, infile, READONLY, &status) ) {
    headas_chat(1, "*** Failed to open %s\n", infile);
    return(status);
  }
  
  inputARF = (struct ARF *) malloc(sizeof(struct ARF));
  if ( (status = ReadARF(inptr, 1, inputARF)) ) {
    headas_chat(1, "*** Failed to read ARF from %s\n", infile);
    return(status);
  }
    

  /* set up binning factor structure */

  binning = (struct BinFactors *) malloc(sizeof(struct BinFactors));

  if ( strcmp(binfile,"") != 0 ) {
    if ( (status = SPReadBinningFile(binfile, binning)) ) {
      headas_chat(1, "*** Failed to read binning data from %s\n", binfile);
      return(status);
    }
  } else {
    binning->StartBin = (long *) malloc(sizeof(long));
    binning->EndBin   = (long *) malloc(sizeof(long));
    binning->Binning  = (long *) malloc(sizeof(long));
    binning->NumberBinFactors = 1;
    binning->StartBin[0] = 1;
    binning->EndBin[0] = inputARF->NumberEnergyBins;
    binning->Binning[0] = binfact;
  }

  /* calculate grouping array */

  groupArray = (int *) malloc(inputARF->NumberEnergyBins*sizeof(int));
  if ( (status = SPSetGroupArray(inputARF->NumberEnergyBins, binning, groupArray)) ) {
    headas_chat(1, "*** Failed to set up grouping array\n");
    return(status);
  }

  /* set up the output ARF */

  outputARF = (struct ARF *) malloc(sizeof(struct ARF));

  n = 0;
  for (i=0; i<inputARF->NumberEnergyBins; i++) {if (groupArray[i] == 1) n++;}
  outputARF->LowEnergy = (float *) malloc(n*sizeof(float));
  outputARF->HighEnergy = (float *) malloc(n*sizeof(float));
  outputARF->EffArea = (float *) malloc(n*sizeof(float));
  outputARF->NumberEnergyBins = n;

  strcpy(outputARF->ARFVersion, inputARF->ARFVersion); 
  strcpy(outputARF->Telescope, inputARF->Telescope); 
  strcpy(outputARF->Instrument, inputARF->Instrument); 
  strcpy(outputARF->Detector, inputARF->Detector); 
  strcpy(outputARF->Filter, inputARF->Filter); 
  strcpy(outputARF->ARFExtensionName, inputARF->ARFExtensionName); 

  /* bin arrays */

  if ( (status = SPBinArray(inputARF->NumberEnergyBins, inputARF->EffArea, groupArray, 2, outputARF->EffArea)) ) {
    headas_chat(1, "*** Failed to bin ARF\n");
    return(status);
  }

  if ( (status = SPBinArray(inputARF->NumberEnergyBins, inputARF->LowEnergy, groupArray, 3, outputARF->LowEnergy)) ) {
    headas_chat(1, "*** Failed to set ENERG_LO values in ARF\n");
    return(status);
  }

  if ( (status = SPBinArray(inputARF->NumberEnergyBins, inputARF->HighEnergy, groupArray, 4, outputARF->HighEnergy)) ) {
    headas_chat(1, "*** Failed to set ENERG_HI values in ARF\n");
    return(status);
  }

  /* create the output ARF file and copy any HDU before the SPECRESP extension in
     the input file */

  headas_clobberfile(outfile);

  if ( fits_create_file(&outptr, outfile, &status) ) {
    headas_chat(1, "*** Failed to create %s\n", outfile);
    return(status);
  }

  if ( fits_copy_file(inptr, outptr, 1, 0, 0, &status) ) {
    headas_chat(1, "*** Failed to copy primary extension from %s to %s\n", infile, outfile);
    return(status);
  }


  /* write the SPECRESP extension */
  /* ALSO NEED TO COPY KEYWORDS */

  if ( (status = WriteARF(outptr, outputARF)) ) {
    headas_chat(1, "*** Failed to write ARF to %s\n", outfile);
    return(status);
  }

  /* copy over keywords to new ARF extension */

  if ( (status = HDcopy_keys(inptr, outptr, 1, 1, &status)) ) {
    headas_chat(1, "*** Failed to ARF extension keywords from %s to %s\n", infile, outfile);
    return(status);
  }

  /* copy over any other extensions */

  if ( fits_copy_file(inptr, outptr, 0, 0, 1, &status) ) {
    headas_chat(1, "*** Failed to copy extensions from %s to %s\n", infile, outfile);
    return(status);
  }

  if ( fits_close_file(inptr, &status) ) {
    headas_chat(1, "*** Failed to close %s\n", infile);
    return(status);
  }
  if ( fits_close_file(outptr, &status) ) {
    headas_chat(1, "*** Failed to close %s\n", outfile);
    return(status);
  }

  return(status);
}
